@@ -32,6 +32,10 @@ extern "C" {
 #include <groonga.h>
 #include "mrn_sys.h"
 
+#ifdef MARIADB_BASE_VERSION
+#  define MRN_MARIADB_P 1
+#endif
+
 #if (MYSQL_VERSION_ID >= 50603) || \
     (MYSQL_VERSION_ID >= 50513 && MYSQL_VERSION_ID < 50600) || \
     (MYSQL_VERSION_ID >= 50158 && MYSQL_VERSION_ID < 50500)
@@ -43,7 +47,8 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_ADD_INDEX 1
 #endif
 
-#if MYSQL_VERSION_ID >= 50600
+#if (MYSQL_VERSION_ID >= 50600) || \
+    (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 50302)
 #  define MRN_HANDLER_HAVE_HA_CLOSE 1
 #  define MRN_HANDLER_HAVE_HA_RND_NEXT 1
 #  define MRN_HANDLER_HAVE_HA_RND_POS 1
@@ -57,18 +62,16 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_MULTI_RANGE_READ 1
 #endif
 
+#if MYSQL_VERSION_ID >= 50512
+#  define MRN_HANDLER_HAVE_HA_INPLACE_INDEX_CHANGE
+#endif
+
 #if MYSQL_VERSION_ID < 50600
   typedef Item COND;
 #endif
 
-#ifndef GRN_GEO_DEGREE2MSEC
-#  define GRN_GEO_DEGREE_ACCURACY 10000000
-#  define GRN_GEO_DEGREE2MSEC(degree)\
-    ((int)(((int)(degree) * 60 * 60 * 1000) +\
-           (((long)((degree) * GRN_GEO_DEGREE_ACCURACY) % GRN_GEO_DEGREE_ACCURACY) * 60 / GRN_GEO_DEGREE_ACCURACY * 60 * 1000) +\
-           (((long)((degree) * GRN_GEO_DEGREE_ACCURACY) % GRN_GEO_DEGREE_ACCURACY) * 60 % GRN_GEO_DEGREE_ACCURACY * 60 / 10000)))
-#  define GRN_GEO_MSEC2DEGREE(msec)\
-    ((((int)(msec)) / 3600.0) * 0.001)
+#if MYSQL_VERSION_ID < 50603
+  typedef MYSQL_ERROR Sql_condition;
 #endif
 
 class ha_mroonga;
@@ -123,15 +126,21 @@ public:
 private:
   grn_ctx *ctx;
 
+  grn_obj  encoded_key_buffer;
+  grn_obj  old_value_buffer;
+  grn_obj  new_value_buffer;
+
   grn_obj *grn_table;
   grn_obj **grn_columns;
 
   grn_obj **grn_index_tables;
   grn_obj **grn_index_columns;
 
-  grn_obj *result_geo;
+  grn_obj top_left_point;
+  grn_obj bottom_right_point;
   grn_table_cursor *cursor;
   grn_table_cursor *index_table_cursor;
+  grn_obj *geo_cursor;
   grn_obj *score_column;
   grn_obj *key_accessor;
 
@@ -328,11 +337,14 @@ protected:
 private:
   void push_warning_unsupported_spatial_index_search(enum ha_rkey_function flag);
   void clear_cursor();
+  void clear_cursor_geo();
   void clear_search_result();
+  void clear_search_result_geo();
   grn_obj *find_tokenizer(const char *name, int name_length);
+  int wrapper_get_next_record(uchar *buf);
   int storage_get_next_record(uchar *buf);
-  grn_obj *storage_geo_select_in_rectangle(grn_obj *index_column,
-                                           const uchar *rectangle);
+  void geo_store_rectangle(const uchar *rectangle);
+  int generic_geo_open_cursor(const uchar *key, enum ha_rkey_function find_flag);
 
 #ifdef MRN_HANDLER_HAVE_HA_CLOSE
   int close();
@@ -348,18 +360,29 @@ private:
   int storage_create(const char *name, TABLE *table,
                      HA_CREATE_INFO *info, MRN_SHARE *tmp_share);
   int wrapper_validate_key_info(KEY *key_info);
-  int wrapper_create_index_table(grn_obj *grn_table,
-                                 const char *grn_table_name,
-                                 int i,
-                                 KEY *key_info,
-                                 grn_obj **index_tables,
-                                 MRN_SHARE *tmp_share);
+  int wrapper_create_index_fulltext(grn_obj *grn_table,
+                                    const char *grn_table_name,
+                                    int i,
+                                    KEY *key_info,
+                                    grn_obj **index_tables,
+                                    MRN_SHARE *tmp_share);
+  int wrapper_create_index_geo(grn_obj *grn_table,
+                               const char *grn_table_name,
+                               int i,
+                               KEY *key_info,
+                               grn_obj **index_tables,
+                               MRN_SHARE *tmp_share);
   int wrapper_create_index(const char *name, TABLE *table,
                            HA_CREATE_INFO *info, MRN_SHARE *tmp_share);
   int storage_create_validate_pseudo_column(TABLE *table);
   int storage_create_validate_index(TABLE *table);
   int storage_create_index(TABLE *table, const char *grn_table_name,
-                           grn_obj *grn_table, MRN_SHARE *tmp_share);
+                           grn_obj *grn_table, MRN_SHARE *tmp_share,
+                           KEY *key_info, grn_obj **index_tables,
+                           grn_obj **index_columns, uint i);
+  int storage_create_indexes(TABLE *table, const char *grn_table_name,
+                             grn_obj *grn_table, MRN_SHARE *tmp_share);
+  int close_databases();
   int ensure_database_create(const char *name);
   int ensure_database_open(const char *name);
   int wrapper_delete_table(const char *name, MRN_SHARE *tmp_share,
@@ -395,11 +418,14 @@ private:
   void storage_start_bulk_insert(ha_rows rows);
   int wrapper_end_bulk_insert();
   int storage_end_bulk_insert();
-  bool wrapper_have_fulltext_index();
+  bool wrapper_is_target_index(KEY *key_info);
+  bool wrapper_have_target_index();
   int wrapper_write_row(uchar *buf);
   int wrapper_write_row_index(uchar *buf);
   int storage_write_row(uchar *buf);
-  int storage_write_row_index(uchar *buf, grn_id record_id);
+  int storage_write_row_index(uchar *buf, grn_id record_id,
+                              KEY *key_info, grn_obj *index_column);
+  int storage_write_row_indexes(uchar *buf, grn_id record_id);
   int wrapper_get_record_id(uchar *data, grn_id *record_id, const char *context);
   int wrapper_update_row(const uchar *old_data, uchar *new_data);
   int wrapper_update_row_index(const uchar *old_data, uchar *new_data);
@@ -431,7 +457,7 @@ private:
                                    key_range *range_max);
   ha_rows storage_records_in_range(uint key_nr, key_range *range_min,
                                    key_range *range_max);
-  ha_rows storage_records_in_range_geo(uint key_nr, key_range *range_min,
+  ha_rows generic_records_in_range_geo(uint key_nr, key_range *range_min,
                                        key_range *range_max);
   int wrapper_index_init(uint idx, bool sorted);
   int storage_index_init(uint idx, bool sorted);
@@ -443,9 +469,6 @@ private:
   int storage_index_read_map(uchar *buf, const uchar *key,
                              key_part_map keypart_map,
                              enum ha_rkey_function find_flag);
-  int storage_index_read_map_geo(uchar *buf, const uchar *key,
-                                 Field *field,
-                                 enum ha_rkey_function find_flag);
   int wrapper_index_read_last_map(uchar *buf, const uchar *key,
                                   key_part_map keypart_map);
   int storage_index_read_last_map(uchar *buf, const uchar *key,
@@ -586,6 +609,8 @@ private:
                         handler_add_index **add);
   int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
                         handler_add_index **add);
+  int storage_add_index_multiple_columns(KEY *key_info, uint num_of_keys,
+                                         grn_obj **index_columns);
   int wrapper_final_add_index(handler_add_index *add, bool commit);
   int storage_final_add_index(handler_add_index *add, bool commit);
   int wrapper_prepare_drop_index(TABLE *table_arg, uint *key_num,
