@@ -18,29 +18,9 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-/* We need to undefine them because my_config.h defines them. :< */
-#  undef VERSION
-#  undef PACKAGE
-#  undef PACKAGE_BUGREPORT
-#  undef PACKAGE_NAME
-#  undef PACKAGE_STRING
-#  undef PACKAGE_TARNAME
-#  undef PACKAGE_VERSION
-#endif
+#include "mrn_mysql.h"
 
-#define MYSQL_SERVER 1
-#include "mysql_version.h"
-
-#if MYSQL_VERSION_ID < 50500
-#  include <mysql_priv.h>
-#  include <mysql/plugin.h>
-#else
-#  include <sql_priv.h>
-#  include <sql_class.h>
-#  include <probes_mysql.h>
-#  include <sql_partition.h>
+#if MYSQL_VERSION_ID >= 50500
 #  include <sql_servers.h>
 #  include <sql_base.h>
 #endif
